{% load whitebox %}

<slot name="video"
      data-config-url="{% tagged_static "whitebox_plugin_device_insta360/insta360_x4/wizard_step_3.mp4" %}">
</slot>

<p class="text-high-emphasis text-lg font-semibold">Step 3 of 4</p>

<p class="text-2xl font-semibold">Camera: Activate</p>

<p class="text-high-emphasis font-medium leading-6">
  To activate your camera, power it on and enable Wi-Fi in settings
  (<span class="font-mono">Settings > Wi-Fi Settings</span>).
</p>

<p class="text-high-emphasis font-medium leading-6">
  Then, download the Insta360 app from the
  <a target="_blank"
     class="text-href"
     href="https://apps.apple.com/rs/app/insta360/id1491299654">
    App Store
  </a>
  on iPhones, or
  <a target="_blank"
     class="text-href"
     href="https://www.insta360.com/download/hot-download">
    Insta360 website
  </a>
  on Androids. Follow the app’s prompts to connect to your camera’s Wi-Fi
  and complete the pairing process. The app will
  also guide you through any necessary firmware updates.
</p>
