{% load whitebox %}

<slot name="video"
      data-config-url="{% tagged_static "whitebox_plugin_device_insta360/insta360_x4/wizard_step_1.mp4" %}">
</slot>

<p class="text-high-emphasis text-lg font-semibold">Step 1 of 4</p>

<p class="text-2xl font-semibold">MicroSD: Insert</p>

<p class="text-high-emphasis font-medium leading-6">
  Before you begin recording, make sure to insert a compatible MicroSD card
  into your {{ model_name }}. To insert the card, open the side compartment on the
  camera, remove the battery and gently push the MicroSD card into the slot
  until it clicks into place, and close the compartment securely.
</p>
