{% load whitebox %}

<slot name="image"
      data-config-url="{% tagged_static introduction_step_picture %}"></slot>

<p class="text-high-emphasis text-lg font-semibold">Introduction</p>

<p class="text-2xl font-semibold">{{ model_name }} setup guide</p>

<p class="text-high-emphasis font-medium leading-6">
  Welcome to the {{ model_name }} setup guide!
  Follow these simple steps to get your camera ready for action. Let's get started!
</p>
