{% load whitebox %}

<slot name="video"
      data-config-url="{% tagged_static "whitebox_plugin_device_insta360/insta360_x4/wizard_step_3.mp4" %}">
</slot>

<p class="text-high-emphasis text-lg font-semibold">Step 4 of 4</p>

<p class="text-2xl font-semibold">Wi-Fi: Enter details</p>

<slot name="wizard_field_block"></slot>
